#include <u.h>
#include <libc.h>
#include <bio.h>
#include "xml.h"

void
xml2(Biobuf *bp, char *path, Elem *ep)
{
	Attr *ap;
	char *s;

	for(; ep; ep = ep->next){
		if(ep->pcdata)
			Bprint(bp, "%s/%s=%q\n", path, ep->name, ep->pcdata);
		else
			Bprint(bp, "%s/%s\n", path, ep->name);
		for(ap = ep->attrs; ap; ap = ap->next)
			Bprint(bp, "%s/%s/%s=%q\n", path, ep->name, ap->name, ap->value);
		if(ep->child){
			if((s = smprint("%s/%s", path, ep->name)) == nil)
				sysfatal("no memory %r\n");
			xml2(bp, s, ep->child);
			free(s);
		}
	}
}

static void
usage(void)
{
	fprint(2, "usage: %s [-w] [-n] [file.xml]\n", argv0);
	fprint(2, "  -w   preserve all whitespace\n");
	fprint(2, "  -n   strip namespace specs\n");
	exits("usage");
}


void
main(int argc, char **argv)
{
	Xml *xp;
	Biobuf out, *bp;
	int i, flags;
	char err[ERRMAX];

	flags = Fcrushwhite;
	ARGBEGIN{
	case 'd':
		xml_debug++;
		break;
	case 'w':
		flags &= ~Fcrushwhite;		/* allow whitespace */
		break;
	case 'n':
		flags |= Fstripnamespace;	/* drop namespaces */
		break;
	}ARGEND

	fmtinstall('q', quotestrfmt);

	Binit(&out, 1, OWRITE);
	if(argc == 0){
		while((xp = xml_parse(0, 8192, flags)) != nil){
			xml2(&out, "", xp->root);
			xml_free(xp);
		}
		rerrstr(err, nelem(err));
		if(*err)
			sysfatal("xml parse failed, %s\n", err);
	}
	else
	for(i = 0; i < argc; i++){
		if((bp = Bopen(argv[i], OREAD)) == nil){
			fprint(2, "%s: %s cannot open %r\n", argv0, argv[i]);
			continue;
		}
		while((xp = xml_Bparse(bp, 8192, flags)) != nil){
			xml2(&out, "", xp->root);
			xml_free(xp);
		}
		rerrstr(err, nelem(err));
		if(*err)
			sysfatal("xml parse failed, %s\n", err);
		Bterm(bp);
	}
	Bterm(&out);
	exits(nil);
}
